package COM.objectspace.jgl;

public final class Finding {
   private Finding() {
   }

   public static InputIterator find(InputIterator var0, InputIterator var1, Object var2) {
      InputIterator var3 = (InputIterator)var0.clone();

      while(!var3.equals(var1) && !var3.get().equals(var2)) {
         var3.advance();
      }

      return var3;
   }

   public static InputIterator find(Container var0, Object var1) {
      return find(var0.start(), var0.finish(), var1);
   }

   public static InputIterator findIf(InputIterator var0, InputIterator var1, UnaryPredicate var2) {
      InputIterator var3 = (InputIterator)var0.clone();

      while(!var3.equals(var1) && !var2.execute(var3.get())) {
         var3.advance();
      }

      return var3;
   }

   public static InputIterator findIf(Container var0, UnaryPredicate var1) {
      return findIf(var0.start(), var0.finish(), var1);
   }

   public static InputIterator adjacentFind(InputIterator var0, InputIterator var1) {
      return adjacentFind(var0, var1, new EqualTo());
   }

   public static InputIterator adjacentFind(Container var0) {
      ForwardIterator var1 = var0.start();
      ForwardIterator var2 = var0.finish();
      return adjacentFind(var1, var2, new EqualTo());
   }

   public static InputIterator adjacentFind(InputIterator var0, InputIterator var1, BinaryPredicate var2) {
      if (var0.equals(var1)) {
         return var1;
      } else {
         InputIterator var3 = (InputIterator)var0.clone();
         InputIterator var4 = (InputIterator)var0.clone();
         var4.advance();

         while(!var4.equals(var1)) {
            if (var2.execute(var3.get(), var4.get())) {
               return var3;
            }

            var3.advance();
            var4.advance();
         }

         return var4;
      }
   }

   public static InputIterator adjacentFind(Container var0, BinaryPredicate var1) {
      return adjacentFind(var0.start(), var0.finish(), var1);
   }

   public static Object detect(ForwardIterator var0, ForwardIterator var1, UnaryPredicate var2) {
      Class var3 = var0.getContainer().getClass();
      Class var4 = var1.getContainer().getClass();
      if (var3 != var4) {
         throw new IllegalArgumentException("iterator containers must be the same");
      } else {
         ForwardIterator var5 = (ForwardIterator)var0.clone();

         while(!var5.equals(var1)) {
            Object var6 = var5.nextElement();
            if (var2.execute(var6)) {
               return var6;
            }
         }

         return null;
      }
   }

   public static Object detect(Container var0, UnaryPredicate var1) {
      return detect(var0.start(), var0.finish(), var1);
   }

   public static boolean some(ForwardIterator var0, ForwardIterator var1, UnaryPredicate var2) {
      return detect(var0, var1, var2) != null;
   }

   public static boolean some(Container var0, UnaryPredicate var1) {
      ForwardIterator var2 = var0.start();
      ForwardIterator var3 = var0.finish();
      return detect(var2, var3, var1) != null;
   }

   public static boolean every(ForwardIterator var0, ForwardIterator var1, UnaryPredicate var2) {
      Class var3 = var0.getContainer().getClass();
      Class var4 = var1.getContainer().getClass();
      if (var3 != var4) {
         throw new IllegalArgumentException("iterator containers must be the same");
      } else {
         ForwardIterator var5 = (ForwardIterator)var0.clone();

         while(!var5.equals(var1)) {
            if (!var2.execute(var5.nextElement())) {
               return false;
            }
         }

         return true;
      }
   }

   public static boolean every(Container var0, UnaryPredicate var1) {
      return every(var0.start(), var0.finish(), var1);
   }
}
