package COM.objectspace.jgl;

public final class Sorting {
   static final int stlThreshold = 16;
   Sequence base;
   BinaryPredicate comparator;

   private Sorting() {
   }

   public static void sort(ForwardIterator var0, ForwardIterator var1) {
      HashComparator var2 = new HashComparator();
      new Sorting(var0, var1, var2);
   }

   public static void sort(ForwardIterator var0, ForwardIterator var1, BinaryPredicate var2) {
      new Sorting(var0, var1, var2);
   }

   public static void sort(Sequence var0) {
      ForwardIterator var1 = var0.start();
      ForwardIterator var2 = var0.finish();
      HashComparator var3 = new HashComparator();
      new Sorting(var1, var2, var3);
   }

   public static void sort(Sequence var0, BinaryPredicate var1) {
      ForwardIterator var2 = var0.start();
      ForwardIterator var3 = var0.finish();
      new Sorting(var2, var3, var1);
   }

   private Sorting(ForwardIterator var1, ForwardIterator var2, BinaryPredicate var3) {
      if (var1.getContainer() instanceof Sequence && var2.getContainer() instanceof Sequence) {
         this.base = (Sequence)var1.getContainer();
         this.comparator = var3;
         int var4 = this.base.start().distance(var1);
         int var5 = this.base.start().distance(var2);
         this.quickSortLoop(var4, var5);
         this.finalInsertionSort(var4, var5);
      } else {
         throw new IllegalArgumentException("iterator containers must be a Sequence");
      }
   }

   void finalInsertionSort(int var1, int var2) {
      if (var2 - var1 > 16) {
         int var6 = var1 + 16;

         for(int var4 = var1 + 1; var4 < var6; ++var4) {
            this.linearInsert(var1, var4);
         }

         for(int var5 = var6; var5 < var2; ++var5) {
            this.unguardedLinearInsert(var5, this.base.at(var5));
         }

      } else {
         for(int var3 = var1 + 1; var3 < var2; ++var3) {
            this.linearInsert(var1, var3);
         }

      }
   }

   void unguardedLinearInsert(int var1, Object var2) {
      int var3 = var1 - 1;

      while(this.comparator.execute(var2, this.base.at(var3))) {
         this.base.put(var1--, this.base.at(var3--));
      }

      this.base.put(var1, var2);
   }

   void linearInsert(int var1, int var2) {
      Object var3 = this.base.at(var2);
      if (!this.comparator.execute(var3, this.base.at(var1))) {
         this.unguardedLinearInsert(var2, var3);
      } else {
         for(int var4 = var2; var4 > var1; --var4) {
            this.base.put(var4, this.base.at(var4 - 1));
         }

         this.base.put(var1, var3);
      }
   }

   void quickSortLoop(int var1, int var2) {
      while(var2 - var1 > 16) {
         Object var4 = this.base.at(var1);
         Object var5 = this.base.at(var1 + (var2 - var1) / 2);
         Object var6 = this.base.at(var2 - 1);
         Object var3;
         if (this.comparator.execute(var4, var5)) {
            if (this.comparator.execute(var5, var6)) {
               var3 = var5;
            } else if (this.comparator.execute(var4, var6)) {
               var3 = var6;
            } else {
               var3 = var4;
            }
         } else if (this.comparator.execute(var4, var6)) {
            var3 = var4;
         } else if (this.comparator.execute(var5, var6)) {
            var3 = var6;
         } else {
            var3 = var5;
         }

         int var7 = var1;
         int var8 = var2;

         while(true) {
            while(this.comparator.execute(this.base.at(var7), var3)) {
               ++var7;
            }

            --var8;

            while(this.comparator.execute(var3, this.base.at(var8))) {
               --var8;
            }

            if (var7 >= var8) {
               if (var7 - var1 >= var2 - var7) {
                  this.quickSortLoop(var7, var2);
                  var2 = var7;
                  break;
               }

               this.quickSortLoop(var1, var7);
               var1 = var7;
               break;
            }

            Object var9 = this.base.at(var7);
            this.base.put(var7++, this.base.at(var8));
            this.base.put(var8, var9);
         }
      }

   }
}
